@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2002 Toshikaz Hirabayashi
+// Copyright (C) 1999-2004 Toshikaz Hirabayashi
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -30,6 +30,7 @@
 #include <WSCroot.h>
 #include <WSDdev.h>
 #include <WSClistData.h>
+#include <WSCindexData.h>
 #include <WSCstring.h>
 class WSCbase;
 
@@ -45,6 +46,13 @@ class WSDappDev   :public WSDdev{
     long      _policy;
     WSCbase*  _focus_dialog;
     WSClistData _pld_menu_list;
+    long      _mode; 
+    void*     _alt_context; 
+    void*     _alt_context2; 
+    WSCindexData _ext_policy_list;
+    WSCstring _ext_policy_name;
+    WSCindexData* _current_policy_fn_list;
+
 
     static WSDappDev*   _instance;
     static WSClistData* _init_handler;
@@ -101,6 +109,20 @@ class WSDappDev   :public WSDdev{
     int getArgc();
     char** getArgv();
     void exit(long);
+    void setMode(long);
+    long getMode();
+    void setContext(void*);
+    void* getContext();
+    void setContext2(void*);
+    void* getContext2();
+
+    WSCstring getExtGuiPolicyName();
+    WSCbool getExtGuiPolicySelected();
+    void setExtGuiPolicy(char* policy_name,WSCbool create_event);
+    void setExtGuiPolicyProc(char* policy_name,char* procname,void(*)(void*));
+    void* getExtGuiPolicyProc(char* procname);
+    WSCindexData* getExtGuiPolicyList();
+
     WSMFclassDef(WSDappDev,WSDdev);
 };
 extern WSDLEX WSDappDev*  WSGIappDev();
