@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2002 Toshikaz Hirabayashi
+// Copyright (C) 1999-2004 Toshikaz Hirabayashi
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -117,6 +117,7 @@ extern void operator delete[](void*);
 #define WS_TYPE_WINDOW   0x0001
 #define WS_TYPE_MANAGER  0x0002
 #define WS_TYPE_FORM     0x0004
+#define WS_TYPE_SCR_FORM 0x0008
 
 #define WS_OOF_SUFFIX   ".oof"
 #define WS_OOF_SUFFIX2  "oof"
@@ -305,6 +306,22 @@ typedef void (*WSCeventProc)(WSCbase*);
 #ifndef True
 #define True        (WSCbool)1
 #endif
+
+#else //TE
+extern "C" {
+extern char*  getcwd(char*,int);
+extern int  chdir(const char*);
+//struct timeval{
+//    int tv_sec;        /* Seconds.  */
+//    int tv_usec;       /* Microseconds.  */
+//};
+struct timezone {
+    int tz_minuteswest;     /* Minutes west of GMT.  */
+    int tz_dsttime;     /* Nonzero if DST is ever in effect.  */
+};
+extern int gettimeofday(struct timeval *__tv,
+                  struct timezone *__tz);
+};
 #endif //TE
 
 //property type
@@ -365,6 +382,8 @@ typedef void (*WSCeventProc)(WSCbase*);
 #define  WSEV_SPECIAL_FOCUS_CH    (long)48
 #define  WSEV_CHILD_ADDED         (long)49
 #define  WSEV_SELECTION_CH        (long)50
+#define  WSEV_SCR_MOUSE_DOUBLE_CLICK  (long)51
+#define  WSEV_ITEM_DOUBLE_CLICKED    (long)52
 
 #define  WSEV_NONE_BIT                0x00000000
 #define  WSEV_INITIALIZE_BIT          0x00000001
@@ -409,7 +428,7 @@ return _win_main(hInstance,hPrevInstance,lpCmdLine,nCmdShow); }; };
 #include <math.h>
 #include <stdarg.h>
 
-#ifdef MSW
+#if defined MSW
 extern int _dbprintf_init();
 extern int _dbprintf(const char*,...);
 #define dbprintf _dbprintf
@@ -628,8 +647,8 @@ extern WSDLEX void(* WSGFgetDeleteHandler() )(WSCbase*) ;
 extern WSDLEX void          WSGFsetCreateHandler(void (*)(WSCbase*));
 extern WSDLEX void          WSGFexecCreateHandler(WSCbase*);
 extern WSDLEX char*         WSGFtriggerToName(long ev);
-extern WSDLEX char*         WSGFgetFileName(char*);
-extern WSDLEX char*         WSGFgetDirName(char*);
+extern WSDLEX WSCstring     WSGFgetFileName(char*);
+extern WSDLEX WSCstring     WSGFgetDirName(char*);
 extern WSDLEX void          WSGFgetAllPropertyList(WSCbase*,WSClistData*);
 extern WSDLEX long          WSGFloadWindow(char* field,char* fname,WSCbase** pic,WSCbase*);
 extern WSDLEX long          WSGFsaveWindow(char* field,char* fname,WSCbase* pic);
@@ -703,6 +722,7 @@ extern WSDLEX void WSGFwsIePluginInitialize();
 #define WS_POLICY_WINDOWS    2
 #define WS_POLICY_GRADATION  4
 #define WS_POLICY_AQUA       8
+#define WS_POLICY_OTHER     16
 
 #define WS_DATA_SOURCE_NONE      0
 #define WS_DATA_SOURCE_INSTANCE  1
@@ -867,4 +887,36 @@ extern WSDLEX long WSGFgetLocaleValue(char*);
 #define WS_LANG_PYTHON 2
 #define WS_LANG_PERL   3
 
+
+//MOUSE
+#define WS_DEFAULT_MOUSE_ID "0   132        0          black      white"
+
+#ifdef WIN32
+#ifdef WS_DEFAULT_MOUSE_ID
+#undef WS_DEFAULT_MOUSE_ID
+#endif
+#define WS_DEFAULT_MOUSE_ID "0   IDC_ARROW"
+#endif
+
+
+#ifdef BTRON
+#ifdef WS_DEFAULT_MOUSE_ID
+#undef WS_DEFAULT_MOUSE_ID
+#endif
+#define WS_DEFAULT_MOUSE_ID "0   PS_SELECT black white"
+#endif
+
+#ifdef TE
+#ifdef WS_DEFAULT_MOUSE_ID
+#undef WS_DEFAULT_MOUSE_ID
+#endif
+#define WS_DEFAULT_MOUSE_ID "0   PS_SELECT black white"
+#endif
+
+#define WS_IMAGE_DEFAULT  (0)
+#define WS_IMAGE_XPM      (1)
+#define WS_IMAGE_BMP      (2)
+#define WS_IMAGE_PNG      (3)
+#define WS_IMAGE_JPG      (4)
+
 #endif //WSCOMH
