@@ -1908,7 +1908,7 @@ void WSCinspector::setLoadProjectHandler(void(*fn)(WSCinspector*,char*)){
   _load_project_fn = fn;
 }
 void WSCinspector::setSelectIndexTab(long no){
-  setProperty(WSNvalue,no);
+  _area2->setProperty(WSNvalue,no);
   setFormVisible(3,True);
 }
 WSCbase* WSCinspector::getDirTree(){
