@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2002 Toshikaz Hirabayashi
+// Copyright (C) 1999-2004 Toshikaz Hirabayashi
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -30,6 +30,7 @@
 #include <WSCdevice.h>
 #include <WSDdev.h>
 #include <WSDappDev.h>
+#include <WSCbaseList.h>
 
 WSMFguiClassInitialize(WSCform,WSCbase);
 WSMFversion(WSCform,WSCbase);
@@ -71,6 +72,7 @@ WSCform::WSCform(WSCbase* base,char* objname):WSCbase(base,objname){
   _a1_color = -1;
   _a2_color = -1;
   _a3_color = -1;
+  WSGIappObjectList()->addEvent(this,WSEV_GUI_POLICY_CH);
 
   WSMFpropertyCreateStart
     WSMFparentCheckVerSrc(WSCform);
@@ -230,6 +232,9 @@ void WSCform::setWorkWSNwidth(WSCushort data){
       setAbsoluteDraw(True);
       needUpdate();
     }
+  }else{
+    _geometry_changed = True;
+    _update_resize = True;
   }
 }
 void WSCform::getWorkWSNwidth(WSCushort* ){}
@@ -251,6 +256,9 @@ void WSCform::setWorkWSNheight(WSCushort data){
       setAbsoluteDraw(True);
       needUpdate();
     }
+  }else{
+    _geometry_changed = True;
+    _update_resize = True;
   }
 }
 void WSCform::getWorkWSNheight(WSCushort* ){}
@@ -476,6 +484,8 @@ void WSCform::onResize(WSCrect* area){
   }
 }
 WSCform::~WSCform(){
+  WSGIappObjectList()->delEvent(this,WSEV_GUI_POLICY_CH);
+
   WSClistData* ghlist = (WSClistData*)getUserData("GH-LIST");
   if (ghlist != NULL){
     delete ghlist;
@@ -498,7 +508,7 @@ long WSCform::_device_initialize(){
   if (dev == NULL){
     return WS_ERR;
   }
-  char pstyle = _pixmap_style;
+  char pstyle = getPixmapStyle();
   dev->setVisible(False);
   if (getXOffsetPtr() == NULL){
     dev->setValue(WSDEV_X,(void*)&_x);
@@ -529,45 +539,70 @@ long WSCform::getObjectType(){
   return (WS_TYPE_MANAGER | WS_TYPE_FORM);
 }
 long WSCform::draw(){
+dbprintf("WSCform::draw %s:%d %s begin..\n",__FILE__,__LINE__,getInstanceName());
   if (_no_refresh != False){
     WSCbase::update();
     setAbsoluteDraw(False);
+dbprintf("WSCform::draw %s:%d %s no refresh..\n",__FILE__,__LINE__,getInstanceName());
     return WS_NO_ERR;
   }
   if (getVisible() == False){
+dbprintf("WSCform::draw %s:%d %s vis=False..\n",__FILE__,__LINE__,getInstanceName());
     return WS_NO_ERR;
   }
   WSDdev* dev = getowndev();
   if (dev == NULL){
+dbprintf("WSCform::draw %s:%d %s dev=NULL..\n",__FILE__,__LINE__,getInstanceName());
     return WS_ERR;
   }
 
   WSCbool absolute = getAbsoluteDraw(); 
   if (absolute == False  ){
-#if 0
-    if (getDotMode() == False && _grad_no == 0){
-      if ( dev->isExposed(0,0,_w,_shadow_thick) == False &&
-           dev->isExposed(0,0,_shadow_thick,_h) == False &&
-           dev->isExposed(_w -_shadow_thick,0,_shadow_thick,_h) == False &&
-           dev->isExposed(0,_h -_shadow_thick,_w,_shadow_thick) == False ){
-        return WS_NO_ERR;
-      }
-    }else{
-#endif
-      if ( dev->isExposed(0,0,_w,_h) == False ){
-        return WS_NO_ERR;
-      }
-#if 0
+    if ( dev->isExposed(0,0,_w,_h) == False ){
+dbprintf("WSCform::draw %s:%d %s not exposed..\n",__FILE__,__LINE__,getInstanceName());
+      return WS_NO_ERR;
     }
-#endif
+  }
+  if (_update_resize != False){
+    _update_resize = False;
+    execResizeEvent();
   }
 
   if (_geometry_changed != False){
+dbprintf("WSCform::draw %s:%d %s call _exec_child_adjust_for_anchors()..\n",__FILE__,__LINE__,getInstanceName());
     _exec_child_adjust_for_anchors();
   }
 
+  if (WSGIappDev()->getExtGuiPolicySelected() != False){
+    if (_ext_draw_proc == NULL){
+      _ext_draw_proc = (void(*)(void*))WSGIappDev()->
+                               getExtGuiPolicyProc("WSCform::draw");
+    }
+    if (_ext_draw_proc != NULL){
+      WSCform_draw_struct st;
+      st._this = this;
+      st.dev = dev;
+      st.x = _x;
+      st.y = _y;
+      st.width = _w;
+      st.height = _h;
+      st.shadow_thickness = _shadow_thick;
+      st.shadow_type = _shadow_type;
+      st.fore_color = _fore_color;
+      st.back_color = _back_color;
+      st.top_shadow_color = _ts_color;
+      st.bottom_shadow_color = _bs_color;
+      st.back_pixmap = _bp_color;
+      st.gradation = _grad_no;
+      st.gradation_margin = _grad_margin;
+      _ext_draw_proc(&st);
+      return WS_NO_ERR;
+    }  
+  }  
+
   long err = dev->beginDraw(0,0,_w,_h,absolute); 
   if (err != WS_NO_ERR){
+dbprintf("WSCform::draw %s:%d %s dev not ready..\n",__FILE__,__LINE__,getInstanceName());
     return WS_NO_ERR;
   }
   short ex = 0;
@@ -625,6 +660,15 @@ void WSCform::_exec_child_adjust_for_anchors(){
       for(i=0; i<num; i++){
         WSCbase* child = (WSCbase*)children[i];
         child->_adjust_for_anchors(_w,_h);
+        if (getPixmapStyle() == WS_DYNAMIC_PIXMAP){
+          if (child->isNeedUpdate() != False){
+            if ((long)child->getUserData("anchor update..") == 0){
+              child->setUserData("anchor update..",(void*)1);
+              child->update();
+              child->setUserData("anchor update..",(void*)0);
+            }
+          }
+        }
       } 
     }
   }
@@ -703,7 +747,7 @@ long WSCform::cdraw(){
   if (getVisible() == False){
     return WS_ERR;
   }
-  if (_pixmap_style != WS_DIRECT_WINDOW){
+  if (getPixmapStyle() != WS_DIRECT_WINDOW){
     redraw();
   }else{
     WSCrect area; 
@@ -753,6 +797,10 @@ void WSCform::_adjust_for_anchors(WSCushort w,WSCushort h){
         }
         _anchor_left_use = True;
         _anchor_right_use = True;
+      }else{
+        if (_anchor_left != _x){
+          setProperty(WSNx,_anchor_left);
+        }
       }
     }
     needUpdate();
@@ -781,6 +829,10 @@ void WSCform::_adjust_for_anchors(WSCushort w,WSCushort h){
         }
         _anchor_top_use = True;
         _anchor_bottom_use = True;
+      }else{
+        if (_anchor_top != _y){
+          setProperty(WSNy,_anchor_top);
+        }
       }
     }
     needUpdate();
@@ -818,3 +870,33 @@ long WSCform::execEventProc(long ev, void* data){
   }
   return WSCbase::execEventProc(ev,data);
 }
+long WSCform::getPixmapStyle(){
+#ifndef BTRON
+#ifndef TE
+  return _pixmap_style;
+#else
+  return WS_DYNAMIC_PIXMAP;
+#endif
+#else
+  return WS_DYNAMIC_PIXMAP;
+#endif
+}
+
+void(*WSCform::_ext_draw_proc)(void*) = NULL;
+WSCstring WSCform::_policy_name;
+
+void WSCform::onGuiPolicyChange(long policy){
+  WSCbase::onGuiPolicyChange(policy);
+  if (WSGIappDev()->getExtGuiPolicySelected() == False){
+    _ext_draw_proc = NULL;
+  }else{
+    if (!strcmp((char*)_policy_name,(char*)WSGIappDev()->getExtGuiPolicyName())){
+    }else{
+      _ext_draw_proc = (void(*)(void*))WSGIappDev()->getExtGuiPolicyProc("WSCform::draw");
+    }
+  }
+  needUpdate();
+  setAbsoluteDraw(True);
+  redraw();
+}
+
