@@ -242,7 +242,13 @@ char* WSClocale::getString(char* index,long encode){
     WSCstring tmp(index);
     if (tmp.isExist(":WSSTR") <0){
     }else{
+      if (!strcmp(getLocaleName(),"UTF8")){
+        WSClocale* locale = WSGIappLocaleSet()->getLocale(WSN_EN_ISO8859_1);
+        if (locale != NULL){
+          return locale->getString(index,encode);
+        }
 fprintf(stderr,"WSClocale::getString locale=%s index=%s not found.\n",getLocaleName(),index);
+      }
     }
     return index;
   }
@@ -251,6 +257,10 @@ fprintf(stderr,"WSClocale::getString locale=%s index=%s not found.\n",getLocaleN
     WSCstring tmp(index);
     if (tmp.isExist(":WSSTR") <0){
     }else{
+      WSClocale* locale = WSGIappLocaleSet()->getLocale(WSN_EN_ISO8859_1);
+      if (locale != NULL){
+        return locale->getString(index,encode);
+      }
 fprintf(stderr,"WSClocale::getString locale=%s index=%s not found.\n",getLocaleName(),index);
     }
     return index;
