@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2002 Toshikaz Hirabayashi
+// Copyright (C) 1999-2004 Toshikaz Hirabayashi
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -30,6 +30,7 @@
 #include <WSDmessage.h>
 #include <WSCbase.h>
 #include <WSDenv.h>
+#include <WSCguiPolicy.h>
 
 WSDappDev*    WSDappDev::_instance = NULL;
 WSMFclassInit(WSDappDev,WSDdev);
@@ -206,6 +207,8 @@ WSCbase* WSDappDev::getSelectionOwner(){
 }
 
 void WSDappDev::setGuiPolicy(long val,WSCbool create_event){
+  WSGIappGuiPolicy()->setGuiPolicy(val,create_event);
+#if 0
   _policy = val;
   if (!(val & WS_POLICY_ORIGINAL) && 
       !(val & WS_POLICY_WINDOWS)){
@@ -219,12 +222,16 @@ void WSDappDev::setGuiPolicy(long val,WSCbool create_event){
   if (create_event != False){
     WSGIappObjectList()->execEvent(WSEV_GUI_POLICY_CH,(void*)_policy);
   }
+#endif
 }
 long WSDappDev::getGuiPolicy(){
+  return WSGIappGuiPolicy()->getGuiPolicy();
+#if 0
   if (_policy == -1){
     return (WS_POLICY_WINDOWS | WS_POLICY_GRADATION);
   }
   return _policy;
+#endif
 }
 
 void WSDappDev::setFocusDialog(WSCbase* client,WSCbool fl){
@@ -272,3 +279,76 @@ void* WSDappDev::getContext2(){
   return _alt_context2;
 }
 
+WSCstring WSDappDev::getExtGuiPolicyName(){
+  return WSGIappGuiPolicy()->getExtGuiPolicyName();
+#if 0
+  return _ext_policy_name;
+#endif
+}
+WSCbool WSDappDev::getExtGuiPolicySelected(){
+  return WSGIappGuiPolicy()->getExtGuiPolicySelected();
+#if 0
+  if (_policy & WS_POLICY_OTHER){
+    return True;
+  }
+  return False;
+#endif
+}
+void WSDappDev::setExtGuiPolicy(char* pname,WSCbool fl){
+  WSGIappGuiPolicy()->setExtGuiPolicy(pname,fl);
+#if 0
+  if (pname == NULL || !strcmp(pname,"")){
+    _ext_policy_name.setString("");
+    _policy &= ~WS_POLICY_OTHER;
+    _current_policy_fn_list = NULL;
+  }else{
+    _ext_policy_name.setString(pname);
+    _policy |= WS_POLICY_OTHER;
+    _current_policy_fn_list = (WSCindexData*)_ext_policy_list[_ext_policy_name.getString()];
+  }
+  if (fl != False){
+    WSGIappObjectList()->execEvent(WSEV_GUI_POLICY_CH,(void*)_policy);
+    WSGIappObjectList()->execUpdate();
+  }
+#endif
+}
+void WSDappDev::setExtGuiPolicyProc(char* plname,char* pname,void(*fn)(void*)){
+  WSGIappGuiPolicy()->setExtGuiPolicyProc(plname,pname,fn);
+#if 0
+  if (plname == NULL || pname == NULL){
+    return;
+  }
+
+  WSCindexData* plist = (WSCindexData*)_ext_policy_list[plname];
+  if (plist == NULL){
+    plist = new WSCindexData;
+    _ext_policy_list[plname] = (void*)plist;
+  }
+  if (plist == NULL){
+    return;
+  }
+  (*plist)[pname] = (void*)fn;
+#endif
+}
+
+void* WSDappDev::getExtGuiPolicyProc(char* pname){
+  return WSGIappGuiPolicy()->getExtGuiPolicyProc(pname);
+#if 0
+  if (getExtGuiPolicySelected() == False){
+    return NULL;
+  }
+  if (_current_policy_fn_list == NULL){
+    _current_policy_fn_list = (WSCindexData*)_ext_policy_list[_ext_policy_name.getString()];
+  }
+  if (_current_policy_fn_list == NULL){
+    return NULL;
+  }
+  return (*_current_policy_fn_list)[pname];
+#endif
+}
+WSCindexData* WSDappDev::getExtGuiPolicyList(){
+  WSGIappGuiPolicy()->getExtGuiPolicyList();
+#if 0
+  return &_ext_policy_list;
+#endif
+}
