@@ -24,6 +24,11 @@
 // dealings in this Software without prior written authorization from
 // Toshikaz Hirabayashi.
 
+#if defined (BTRON)  || defined (TE)
+#include <basic.h>
+#include <btron/clk.h>
+#endif
+
 #include <WScom.h>
 #include <WSDdev.h>
 #include <WSCstring.h>
@@ -88,6 +93,18 @@ int _dbprintf(const char* format,...){
   if (_ws_debug_fp == NULL){
     return -1;
   }
+#if 1 //TEST
+
+  int i;
+  void* ptr[100];
+  for(i=0; i<100;i++){
+    ptr[i] = malloc(i*10);
+    memset(ptr[i],0,i*10);
+  }
+  for(i=0; i<100;i++){
+    delete ptr[i];
+  }
+#endif //TEST
   int ret;
   va_list va;
   va_start(va,format);
@@ -3023,6 +3040,25 @@ void WSGFgettimeofday(long* tv){
   tv[5] = st.wSecond;
   tv[6] = st.wDayOfWeek;
 #endif
+#if defined (BTRON)  || defined (TE)
+  W tim;
+  DATE_TIM tm;
+  TIMEZONE tz;
+  
+  get_tim(&tim,&tz);
+  if ( tz.adjust){
+    get_tod(&tm,tim,(Bool)1);
+  }else{
+    get_tod(&tm,tim,(Bool)0);
+  }
+  tv[0] = tm.d_year + 1900;
+  tv[1] = tm.d_month;
+  tv[2] = tm.d_day;
+  tv[3] = tm.d_hour;
+  tv[4] = tm.d_min;
+  tv[5] = tm.d_sec;
+  tv[6] = tm.d_wday;
+#endif /* BTRON */
 }
 
 void WSGFsetMargin(WSCbase* obj,WSCushort val){
