@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2002 Toshikaz Hirabayashi
+// Copyright (C) 1999-2004 Toshikaz Hirabayashi
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -812,7 +812,9 @@ void WSCcomboBox::_list_ptr_work(WSCbase* list){
       pt.x = px;
       pt.y = py + scrpos;
 //printf("WSCcomboBox::_list_ptr_work pt.x,pt.y=%d,%d\n",pt.x,pt.y);
-      ldev->execEvent(WSEV_MOUSE_PRESS,&pt);
+//      ldev->execEvent(WSEV_MOUSE_PRESS,&pt);
+      cmb->_list->setSelectItemChanged(True);
+      cmb->_list->onActivate();
     }
   }
 }
