@@ -10,8 +10,12 @@
 /*                                                                         */
 /*-------------------------------------------------------------------------*/
 
+#ifdef WS
+#include <stdio.h>
+#else //WS
 #include  <iomanip.h>
 #include  <iostream.h>
+#endif //WS
 #include  <stdlib.h>
 #include  <stdio.h>
 #include  "j3dproc.h"
@@ -64,6 +68,8 @@ int TProcess::CheckError(int err)
     if (err != 0) {
         pJob->ResetGraph();
         pJob->SendMessage(PROC_ALL, M_STOP);
+#ifdef WS
+#else //WS
         cout << '\n';
         cout << " # of Object  : " << dec << pSpace->count_object() << '\n';
         cout << " # of Process : " << dec << pJob->count_process() << '\n';
@@ -138,6 +144,7 @@ int TProcess::CheckError(int err)
             default : cout << "???? \n";
         }
         cout << flush;
+#endif //WS
         return  1;
     } else return  0;
 }
@@ -462,7 +469,7 @@ int TProcess::execute(long &delta)
                     if (_reg < 0) fl |= f_MINUS;
                     break;
                 case 0xF :
-#ifdef WIN32
+#if defined (WIN32) || defined (TE)
                     if (param < 0) srand(-param);
                     else _reg = long(param*double(rand())/_RAND_MAX);
 #else //WIN32
@@ -766,13 +773,19 @@ int TProcess::execute(long &delta)
                    pJob->GetMouse(Reg[RX],Reg[RY],Reg[RZ]);
                    break;
           case 8:
+#ifdef WS
+#else //WS
                    cout << long(Reg[reg1]) << flush;
+#endif //WS
                    break;
           case 9:
                    if (reg1 != 0) param16 = Reg[reg1];
+#ifdef WS
+#else //WS
                    if (param16 == 13) {
                      cout << '\n' <<flush;
                    } else cout << char(param16) << flush;
+#endif //WS
                    break;
           case 0xA:
 #if BYTE_ORDER == BIG_ENDIAN
@@ -783,14 +796,23 @@ int TProcess::execute(long &delta)
                       i++;
                       s[i] = char(DataMem[int(Reg[reg1]) + i/4] >> (8 * (i%4)));
                   } while (s[i]!=0);
+#ifdef WS
+#else //WS
                    cout << s << flush;
+#endif //WS
               }
 #else
+#ifdef WS
+#else //WS
                    cout << (char *)(&DataMem[ int(Reg[reg1]) ]) << flush;
+#endif //WS
 #endif
                    break;
           case 0x0B:
+#ifdef WS
+#else //WS
                    cout << pCode->GetStr(pc) << flush;
+#endif //WS
                    pc = pc + param16;
                    break;
           case 0x0C:
