@@ -8,6 +8,25 @@ typedef unsigned long  WSCulong;
 typedef unsigned char  WSCuchar;
 typedef unsigned int   WSCuint;
 
+/*#ifdef SWIGRUBY*/
+#if 0 /* deleted .. v3.60-2*/
+/* WSCbool to (or from) Ruby's Truth value. */
+%typemap(in) WSCbool {
+  if (($input == Qfalse) || ($input == Qnil)) {
+    $1 = False;
+  } else {
+    $1 = True;
+  }
+}
+%typemap(out) WSCbool {
+  if ($1 == False) {
+    $result = Qfalse;
+  } else {
+    $result = Qtrue;
+  }
+}
+#endif
+
 #define WS_DATA_SOURCE_NONE      0
 #define WS_DATA_SOURCE_INSTANCE  1
 #define WS_DATA_SOURCE_FILE      2
@@ -101,13 +120,80 @@ struct WSCrect {
 
 class WSCstring {
   public:
-    WSCstring(char*);
+    WSCstring(char*, long en = WS_EN_DEFAULT);
+    WSCstring(const WSCvariant&, long en = WS_EN_DEFAULT);
     WSCstring();
     ~WSCstring();
 
     void setString(char*,long en = WS_EN_DEFAULT);
     char* getString(long en = WS_EN_DEFAULT);
+#if defined(SWIGPYTHON) || defined(SWIGRUBY)
+    %extend {
+      char* __str__() { return self->getString(WS_EN_DEFAULT); }
+    }
+#endif
+    long getChars();
+    long getEncoding();
+    long isExist(const char*,long encode = WS_EN_DEFAULT );
+
+    void addString(const WSCstring&);
+    void addString(const WSCvariant&,long encode = WS_EN_DEFAULT);
+    void addString(const char*,long encode = WS_EN_DEFAULT);
+    void cutString(WSCulong char_pos);
+    void insertString(WSCulong char_pos,WSCstring&);
+    void deleteChar(WSCulong char_pos);
+    void deleteChars(WSCulong char_pos,WSCulong len);
+    void clear();
+    void delLineFeed();
+    void delString(const char*,long num);
+    void delString(const char*,long num,long encode);
+    void delHeadSpace();
+    void delTailSpace();
+
+    void to_upper();    
+    void to_lower();    
+    
+    long replaceString(const char* srcchar,
+        const char* destchar,long num,long encode = WS_EN_DEFAULT); 
+
+    long getWords();
+    long getWords(const char*,long encode = WS_EN_DEFAULT);
+    long getWordCharPos(long wd);
+    long getWordCharPos(long wd,const char*,long encode = WS_EN_DEFAULT);
+    WSCstring getWord(long wd);
+    WSCstring getWord(long num,const char*,long encode = WS_EN_DEFAULT);
+
+    long _seek_next_line();
+
+    long tell();
+    long seek(long);
+    long seekEnd();
+    long eof();
+    long getLines();
+    WSCstring gets();
+
+#if defined(SWIGPYTHON) || defined(SWIGRUBY)
+    WSCstring operator + (const WSCstring&);
+    WSCstring operator + (const WSCvariant&);
+    WSCstring operator + (const char*);
+    WSCstring& operator << (const WSCstring&);
+    WSCstring& operator << (const WSCvariant&);
+    WSCstring& operator << (const char*);
+    WSCstring& operator << (long);
+    WSCstring& operator << (double);
+#endif
+
+#ifdef SWIGPERL
+    %rename(eq) operator ==;
+#elif SWIGRUBY
+    %extend {
+      bool __eq__(const WSCstring& str){ return (*self == str) ? true : false; }
+    }
+    %ignore operator ==;
+#endif
+    long operator == (const WSCstring&);
 };
+
 class WSCvariant {
   public:
     WSCvariant(long);
@@ -131,6 +217,16 @@ class WSCvariant {
     double getDouble();
     char* getCharPtr();
     WSCbase* getInstancePtr();
+#if defined(SWIGPYTHON) || defined(SWIGRUBY)
+    %extend {
+      long __int__() { return self->getLong(); }
+      double __float__() { return self->getDouble(); }
+      char* __str__() { return self->getCharPtr(); }
+#ifdef SWIGRUBY
+      bool to_bool() { return self->getUnsignedChar() ? true : false; }
+#endif
+    }
+#endif /* defined(SWIGPYTHON) || defined(SWIGRUBY) */
 
     void setValue(long);
     void setValue(char*);
@@ -186,8 +282,6 @@ public:
   void onGuiPolicyChange(long policy);
   void onDefaultColorChange();
 
-
-
   char* getInstanceName();
   void setVisible(WSCbool);
   WSCbool getVisible();
@@ -195,8 +289,11 @@ public:
   WSCvariant getProperty(char*);
   WSCbool setProperty(char*,const WSCvariant&);
   WSCbool setProperty(char*,char*);
-  WSCbool setProperty(char*,int);
+  WSCbool setProperty(char*,long);
   WSCbool setProperty(char*,double);
+#ifdef SWIGRUBY
+  WSCbool setProperty(char*,bool);
+#endif
   long addProcedure(char* pname,char* fname,long);
 
   void setParent(WSCbase*);
@@ -255,6 +352,7 @@ class WSDmouse {
   public:
     WSDmouse();
 
+    %apply short *OUTPUT { short *x, short *y };
     WSCbool getMousePosition(short* x,short* y);
     WSCbool getMousePosition(short* x,short* y,WSCbase*);
 
@@ -533,6 +631,7 @@ public:
   ~WSCgrid();
   void onInputFixed(WSCvariant*,long,long);
 
+    %apply unsigned long *OUTPUT { WSCulong *cx, WSCulong *cy };
     long getCellNo(long px,long py,WSCulong* cx,WSCulong* cy);
 
     long getHColumns();
@@ -550,6 +649,7 @@ public:
     long setCellInputStatus(WSCulong x,WSCulong y,WSCbool fl);
     long getCellInputStatus(WSCulong x,WSCulong y,WSCbool* fl);
 
+    %apply unsigned long *OUTPUT { WSCulong *hpos, WSCulong *vpos };
     long getFocusedCell(WSCulong* hpos,WSCulong* vpos);
     long setFocusedCell(WSCulong hpos,WSCulong vpos);
 };
@@ -965,7 +1065,7 @@ public:
   WSCdirTree(WSCbase*,char*);
   ~WSCdirTree();
   long setSelectedDirName(char*,WSCbool opened = False);
-  char getSelectedDirName();
+  char* getSelectedDirName();
 };
 
 
@@ -982,7 +1082,7 @@ public:
   char* getColorName();
   WSCbool getUsePrivateCell();
   void setUsePrivateCell(WSCbool);
-  long getRGB(long*,long*,long*);
+  long getRGB(long* OUTPUT,long* OUTPUT,long* OUTPUT);
   long setColorName(char* cname);
 };
 
@@ -1135,9 +1235,9 @@ public:
     virtual long clear(long col = WSCSHEET_DEF_COLS, long row = WSCSHEET_DEF_ROWS);
 
 public:
-  long      getRowCount() const { return _rows; }
+  long      getRowCount();
   long      setRowCount(long rows = WSCSHEET_DEF_ROWS);
-  long      getColumnCount() const { return _cols; }
+  long      getColumnCount();
   long      setColumnCount(long cols = WSCSHEET_DEF_COLS);
 
   long      getRowHeight(long row);
@@ -1145,9 +1245,9 @@ public:
   long      getColumnWidth(long col);
   long      setColumnWidth(long col, long width);
    
-  long      getFixedRowCount() const { return _fixedRows; }
+  long      getFixedRowCount();
   long      setFixedRowCount(long fixedRows = WSCSHEET_DEF_FIXEDROWS);
-  long      getFixedColumnCount() const { return _fixedCols; }
+  long      getFixedColumnCount();
   long      setFixedColumnCount(long fixedCols = WSCSHEET_DEF_FIXEDCOLS);
 
   long      getFixedRowHeight(long row);
@@ -1157,50 +1257,45 @@ public:
 
   long      setSheetBkColor(char* cname);
   long      setSheetBkColor(short color);
-  short     getSheetBkColor() const { return _sheetBackColor; }
+  short     getSheetBkColor();
 
   long      setCellDefBkColor(char* cname);
   long      setCellDefBkColor(short color);
-  short     getCellDefBkColor() const { return _cellBackColor; }
+  short     getCellDefBkColor();
 
   long      setCellBkColor(char* cname);
-  %name(setCellBkColorWithPos)
-    long    setCellBkColor(long col, long row, char* cname);
+  long      setCellBkColor(long col, long row, char* cname);
   long      setCellBkColor(short color);
-  %name(setCellBkColorWithPos)
-    long    setCellBkColor(long col, long row, short color);
+  long      setCellBkColor(long col, long row, short color);
   short     getCellBkColor(long col, long row);
 
   long      setCellDefTextColor(char* cname);
   long      setCellDefTextColor(short color);
-  short     getCellDefTextColor() const { return _cellTextColor; }
+  short     getCellDefTextColor();
 
   long      setCellTextColor(char* cname);
-  %name(setCellTextColorWithPos)
-    long    setCellTextColor(long col, long row, char* cname);
+  long      setCellTextColor(long col, long row, char* cname);
   long      setCellTextColor(short color);
-  %name(setCellTextColorWithPos)
-    long    setCellTextColor(long col, long row, short color);
+  long      setCellTextColor(long col, long row, short color);
   short     getCellTextColor(long col, long row);
 
   long      setFixedBkColor(char* cname);
   long      setFixedBkColor(short color);
-  short     getFixedBkColor() const { return _fixedBackColor; }
+  short     getFixedBkColor();
 
   long      setFixedTextColor(char* cname);
   long      setFixedTextColor(short color);
-  short     getFixedTextColor() const { return _fixedTextColor; }
+  short     getFixedTextColor();
 
   long      setGridColor(char* cname);
   long      setGridColor(short color);
-  short     getGridColor() { return _gridColor; }
+  short     getGridColor();
 
   long      setGridLines(short glidLines = WSCSHEET_BOTH);
-  short     getGridLines() const { return _gridLines; }
+  short     getGridLines();
 
   long      setCellGridLines(short glidLines = WSCSHEET_BOTH);
-  %name(setCellGridLinesWithPos)
-    long    setCellGridLines(long col, long row, short glidLines = WSCSHEET_BOTH);
+  long      setCellGridLines(long col, long row, short glidLines = WSCSHEET_BOTH);
   short     getCellGridLines(long col, long row);
 
   long      setText(long text);
@@ -1208,16 +1303,11 @@ public:
   long      setText(const char* text);
   long      setText(const WSCvariant& text);
   long      setText(const WSCstring& text);
-  %name(setTextWithPos)
-    long    setText(long col, long row, long text);
-  %name(setTextWithPos)
-    long    setText(long col, long row, double text);
-  %name(setTextWithPos)
-    long    setText(long col, long row, const char* text);
-  %name(setTextWithPos)
-    long    setText(long col, long row, const WSCvariant& text);
-  %name(setTextWithPos)
-    long    setText(long col, long row, const WSCstring& text);
+  long      setText(long col, long row, long text);
+  long      setText(long col, long row, double text);
+  long      setText(long col, long row, const char* text);
+  long      setText(long col, long row, const WSCvariant& text);
+  long      setText(long col, long row, const WSCstring& text);
   WSCstring getText(long col, long row);
 
   long      setFixedText(long col, long row, long text);
@@ -1227,134 +1317,127 @@ public:
   long      setFixedText(long col, long row, const WSCstring& text);
   WSCstring getFixedText(long col, long row);
 
-  //long      getCellNo(WSCpoint* pt, long* col, long* row);
+  %apply long *OUTPUT { long *col, long *row };
+  long      getCellNo(WSCpoint* pt, long* col, long* row);
 
   long      setFocusCell(long col, long row);
-  //void      getFocusCell(long* col, long* row) { *col = _focusCol; *row = _focusRow; }
-  long      getFocusCol() { return _focusCol; }
-  long      getFocusRow() { return _focusRow; }
+  void      getFocusCell(long* col, long* row);
+  long      getFocusCol();
+  long      getFocusRow();
   
   void      setRow(long row);
-  long      getRow() { return _startRow; }
+  long      getRow();
   void      setStartRow(long row);
-  long      getStartRow() { return (_startRow <= _endRow ? _startRow : _endRow); }
+  long      getStartRow();
   void      setEndRow(long row);
-  long      getEndRow() { return (_startRow > _endRow ? _startRow : _endRow); }
+  long      getEndRow();
 
   void      setCol(long col);
-  long      getCol() { return _startCol; }
+  long      getCol();
   void      setStartCol(long col);
-  long      getStartCol() { return (_startCol <= _endCol ? _startCol : _endCol); }
+  long      getStartCol();
   void      setEndCol(long col);
-  long      getEndCol() { return (_startCol > _endCol ? _startCol : _endCol); }
+  long      getEndCol();
 
   void      setSelectCell(WSCbool val);
-  WSCbool   getSelectCell() { return _selectCell; }
+  WSCbool   getSelectCell();
 
-  void      setEditEnabled(WSCbool val) { _editEnabled = val; }
-  WSCbool   getEditEnabled() { return _editEnabled; }
+  void      setEditEnabled(WSCbool val);
+  WSCbool   getEditEnabled();
 
   long      getFocusCellRect(WSCrect* rect);
   long      getVisibleCellRect(long col, long row, WSCrect* rect);
   long      getCellRect(long col, long row, WSCrect* rect);
 
   long      setTopVisibleRow(long row);
-  long      getTopVisibleRow() { return _topVisibleRow; };
+  long      getTopVisibleRow();
   long      setTopVisibleCol(long col);
-  long      getTopVisibleCol() { return _topVisibleCol; };
+  long      getTopVisibleCol();
 
   long      setCellDefFont(short fontno);
-  short     getCellDefFont() { return _font; }
+  short     getCellDefFont();
 
   long      setCellFont(short fontno);
-  %name(setCellFontWithPos)
-    long    setCellFont(long col, long row, short fontno);
-  short     getCellFont() { return _font; }
-  %name(getCellFontWithPos)
-    short   getCellFont(long col, long row);
+  long      setCellFont(long col, long row, short fontno);
+  short     getCellFont();
+  short     getCellFont(long col, long row);
   
   long      setFixedCellFont(short fontno);
-  short     getFixedCellFont() { return _fixedFont; }
+  short     getFixedCellFont();
 
-  void      setVertAutoNumber(WSCbool val) { _vautoNumber = val; }
-  WSCbool   getVertAutoNumber() { return _vautoNumber; }
-  void      setHorzAutoNumber(WSCbool val) { _hautoNumber = val; }
-  WSCbool   getHorzAutoNumber() { return _hautoNumber; }
+  void      setVertAutoNumber(WSCbool val);
+  WSCbool   getVertAutoNumber();
+  void      setHorzAutoNumber(WSCbool val);
+  WSCbool   getHorzAutoNumber();
 
   long      setCellAlignemnt(char align);
-  %name(setCellAlignemntWithPos)
-    long    setCellAlignemnt(long col, long row, char align);
+  long      setCellAlignemnt(long col, long row, char align);
   char      getCellAlignemnt(long col, long row);
 
   long      setFixedCellAlignemnt(long col, long row, char align);
   char      getFixedCellAlignemnt(long col, long row);
 
   long      setCellLock(WSCbool lock);
-  %name(setCellLockWithPos)
-    long    setCellLock(long col, long row, WSCbool lock);
+  long      setCellLock(long col, long row, WSCbool lock);
   WSCbool   getCellLock(long col, long row);
   
-  WSCushort getClientWidth() { return _clientWidth; }
-  WSCushort getClientHeight() { return _clientHeight; }
+  WSCushort getClientWidth();
+  WSCushort getClientHeight();
   
-  void      setNoRefresh(WSCbool val) { _no_refresh = val; }
-  WSCbool   getNoRefresh() { return _no_refresh; }
+  void      setNoRefresh(WSCbool val);
+  WSCbool   getNoRefresh();
   
   long      insertRow();
-  %name(insertRowWithVal)
-    long    insertRow(long row, long rows);
+  long      insertRow(long row, long rows);
   long      insertCol();
-  %name(insertColWithVal)
-    long    insertCol(long col, long cols);
+  long      insertCol(long col, long cols);
   
   long      deleteRow();
-  %name(deleteRowWithVal)
-    long    deleteRow(long row, long rows);
+  long      deleteRow(long row, long rows);
   long      deleteCol();
-  %name(deleteColWithVal)
-    long    deleteCol(long col, long cols);
+  long      deleteCol(long col, long cols);
 
   /// database 
   long      setDataSource(const char* name);
   long      setDataSource(WSCstring& name);
-  WSCstring getDataSource() { _dbwork = _dataSource; return _dbwork; }
+  WSCstring getDataSource();
 
   long      setTableName(const char* table);
   long      setTableName(WSCstring& table);
-  WSCstring getTableName() { _dbwork = _tableName; return _dbwork; }
+  WSCstring getTableName();
   
   long      setDbItem(const char* item);
   long      setDbItem(WSCstring& item);
-  WSCstring getDbItem() { _dbwork = _dbItem; return _dbwork; }
+  WSCstring getDbItem();
 
   long      setDbSortItem1(const char* item);
   long      setDbSortItem1(WSCstring& item);
-  WSCstring getDbSortItem1() { _dbwork = _dbSortItem1; return _dbwork; }
+  WSCstring getDbSortItem1();
 
   long      setDbSort1(WSCuchar sort);
-  WSCuchar  getDbSort1() { return _dbSort1; }
+  WSCuchar  getDbSort1();
 
   long      setDbSortItem2(const char* item);
   long      setDbSortItem2(WSCstring& item);
-  WSCstring getDbSortItem2() { _dbwork = _dbSortItem2; return _dbwork; }
+  WSCstring getDbSortItem2();
 
   long      setDbSort2(WSCuchar sort);
-  WSCuchar  getDbSort2() { return _dbSort2; }
+  WSCuchar  getDbSort2();
 
   long      setDbSortItem3(const char* item);
   long      setDbSortItem3(WSCstring& item);
-  WSCstring getDbSortItem3() { _dbwork = _dbSortItem3; return _dbwork; }
+  WSCstring getDbSortItem3();
 
   long      setDbSort3(WSCuchar sort);
-  WSCuchar  getDbSort3() { return _dbSort3; }
+  WSCuchar  getDbSort3();
   
   long      setDbFilter(const char* filter);
   long      setDbFilter(WSCstring& filter);
-  WSCstring getDbFilter() { _dbwork = _dbFilter; return _dbwork; }
+  WSCstring getDbFilter();
 
   long      setDbStmtString(const char* stmt);
   long      setDbStmtString(WSCstring& stmt);
-  WSCstring getDbStmtString() { _dbwork = _dbStmtString; return _dbwork; }
+  WSCstring getDbStmtString();
 
   long      dbRefresh();
 };
