@@ -32,7 +32,8 @@
 extern "C" {
 #include <png.h>
 extern long PngReadFileToPixmap(Display* display,Window window,GC gc,
-             char* filename,Pixmap* pixmap,long* w,long* h);
+             char* filename,Pixmap* pixmap,Pixmap* pixmap_mask,long* w,long* h);
+extern long PngWriteBufToFp(FILE* fp, char* buf,long width,long height);
 };
 
 void png_cexcept_error(png_structp png_ptr, png_const_charp msg){
@@ -41,7 +42,8 @@ void png_cexcept_error(png_structp png_ptr, png_const_charp msg){
    }
 }
 
-long PngReadFileToPixmap(Display* display,Window window,GC gc,char* filename,Pixmap* pixmap,long* w,long* h){
+long PngReadFileToPixmap(Display* display,Window window,GC gc,char* filename,Pixmap* pixmap,Pixmap* pixmap_mask,long* w,long* h){
+    *pixmap_mask = (Pixmap)0;
 
     int red_mask, green_mask, blue_mask;
     int red_shift, green_shift, blue_shift;
@@ -92,25 +94,44 @@ long PngReadFileToPixmap(Display* display,Window window,GC gc,char* filename,Pix
     png_uint_32 png_row_bytes = png_get_rowbytes(png_ptr, info_ptr);
     png_uint_32 png_channels = png_get_channels(png_ptr, info_ptr);
 
+//printf("png_row_bytes=%d channels=%d  \n",png_row_bytes,png_channels);
+//printf("depth=%d \n",png_depth);
+
     if (png_depth < 8){
       if (png_color_type == PNG_COLOR_TYPE_GRAY ){
         png_set_gray_1_2_4_to_8(png_ptr);
-        png_row_bytes = png_width;
+        png_row_bytes = png_width * 3;
+        png_channels = 3;
+      }else if (png_color_type == PNG_COLOR_TYPE_GRAY_ALPHA){
+        png_set_gray_1_2_4_to_8(png_ptr);
+        png_row_bytes = png_width * 4;
+        png_channels = 4;
       }else{
         png_set_expand(png_ptr);
-        png_row_bytes = png_width;
         png_row_bytes = png_width * 3;
         png_channels = 3;
       }
     }
     if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS)){
+//printf("PNG_INFO_tRNS\n");
       png_set_expand(png_ptr);
       png_row_bytes = png_width;
+      if (png_channels == 1){
+        png_row_bytes = png_width * 4;
+        png_channels = 4;
+      }
+      if (png_channels == 3){
+        png_row_bytes = png_width * 4;
+        png_channels = 4;
+      }
     }
     if (png_color_type == PNG_COLOR_TYPE_GRAY ||
         png_color_type == PNG_COLOR_TYPE_GRAY_ALPHA){
       png_set_gray_to_rgb(png_ptr);
-      png_row_bytes = png_width;
+/*      png_row_bytes = png_width;*/
+      if (png_channels < 2){
+        png_row_bytes = png_width;
+      }
     }
 
     if (png_color_type == PNG_COLOR_TYPE_PALETTE){
@@ -118,13 +139,24 @@ long PngReadFileToPixmap(Display* display,Window window,GC gc,char* filename,Pix
       png_row_bytes = png_width * 3;
       png_channels = 3;
     }
-
+    if (png_channels == 2){
+      png_row_bytes = png_width * 4;
+      png_channels = 4;
+    }
+    if (png_channels == 1){
+      png_row_bytes = png_width * 3;
+      png_channels = 3;
+    }
     long rwidth = png_width;
     long rheight = png_height;
     long components = png_channels;
 
-    unsigned char* buf =
-        new unsigned char[png_row_bytes * png_height * sizeof(png_byte)];
+    unsigned char* buf = NULL;
+//    if (components == 2){
+//      buf = new unsigned char[png_row_bytes * png_height * sizeof(png_byte)*2];
+//    }else{
+      buf = new unsigned char[png_row_bytes * png_height * sizeof(png_byte)];
+//    }
     if (buf == NULL){
 fprintf(stderr,"png read error: out of memory..\n");
       png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
@@ -140,7 +172,11 @@ fprintf(stderr,"png read error: out of memory..\n");
     }
     int i;
     for(i = 0; i < (int)png_height; i++){
-      png_row_ptrs[i] = (png_byte*)(buf + i * png_row_bytes);
+//      if (components == 2){
+//        png_row_ptrs[i] = (png_byte*)(buf + i * png_row_bytes*2);
+//      }else{
+        png_row_ptrs[i] = (png_byte*)(buf + i * png_row_bytes);
+//      }
     }
     png_read_image(png_ptr, png_row_ptrs);
     png_read_end(png_ptr,NULL);
@@ -166,6 +202,7 @@ fprintf(stderr,"png read error: out of memory..\n");
 
 
     XImage* image = 0;
+    XImage* image_mask = 0;
     Visual* visual = XDefaultVisual(display,XDefaultScreen(display));
     if (display_depth >16){
       image = XCreateImage(display,visual, display_depth,
@@ -178,7 +215,10 @@ fprintf(stderr,"png read error: out of memory..\n");
       image = XCreateImage(display,visual, display_depth,
                            ZPixmap,0,0,vwidth,vheight,8,0);
     }
-
+    if (components == 4){
+      image_mask = XCreateImage(display,visual, 1,
+                           ZPixmap,0,0,vwidth,vheight,8,0);
+    }
     msb_flag = (ImageByteOrder(display) == MSBFirst)?1:0;
 
     if (XGetWindowAttributes(display,
@@ -220,9 +260,15 @@ fprintf(stderr,"png read error: out of memory..\n");
     }
 
     char* data1 = new char[image->bytes_per_line * vheight];
+    char* data2 = NULL;
+    if (components == 4){
+      data2 = new char[image_mask->bytes_per_line * vheight];
+      memset(data2,0,sizeof(char)* image_mask->bytes_per_line * vheight);
+    }
     unsigned char r,g,b;
     long ptr = 0;
     long ptr2 = 0;
+    long ptr3 = 0;
     long j;
     if (image->bits_per_pixel ==32){
       if (components == 3 || components == 4){
@@ -232,12 +278,22 @@ fprintf(stderr,"png read error: out of memory..\n");
               ptr = (long)((double)i*rheight/vheight)*rwidth
                    + (long)((double)j*rwidth/vwidth);
               ptr *= 3;
+              if (components == 4){
+                ptr *= 4;
+              }
             }
             r = buf[ptr++];
             g = buf[ptr++];
             b = buf[ptr++];
             if (components == 4){
-              ptr++;
+              if( buf[ptr++]){
+                ((unsigned char*)data2)[ptr3] |= (1<< (j%8));
+              }
+              if (j%8 == 7){
+                ptr3++;
+              }else if (j == vwidth -1){
+                ptr3++;
+              }
             }
             udat = 0;
             if (red_shift >= 0){
@@ -301,12 +357,22 @@ fprintf(stderr,"png read error: out of memory..\n");
               ptr = (long)((double)i*rheight/vheight)*rwidth
                    + (long)((double)j*rwidth/vwidth);
               ptr *= 3;
+              if (components == 4){
+                ptr *= 4;
+              }
             }
             r = buf[ptr++];
             g = buf[ptr++];
             b = buf[ptr++];
             if (components == 4){
-              ptr++;
+              if( buf[ptr++]){
+                ((unsigned char*)data2)[ptr3] |= (1<< (j%8));
+              }
+              if (j%8 == 7){
+                ptr3++;
+              }else if (j == vwidth -1){
+                ptr3++;
+              }
             }
 
             udat = 0;
@@ -367,12 +433,22 @@ fprintf(stderr,"png read error: out of memory..\n");
               ptr = (long)((double)i*rheight/vheight+0.5)*rwidth
                    + (long)((double)j*rwidth/vwidth + 0.5);
               ptr *= 3;
+              if (components == 4){
+                ptr *= 4;
+              }
             }
             unsigned int rr = buf[ptr++];
             unsigned int gg = buf[ptr++];
             unsigned int bb = buf[ptr++];
             if (components == 4){
-              ptr++;
+              if( buf[ptr++]){
+                ((unsigned char*)data2)[ptr3] |= (1<< (j%8));
+              }
+              if (j%8 == 7){
+                ptr3++;
+              }else if (j == vwidth -1){
+                ptr3++;
+              }
             }
 
             udat = 0;
@@ -459,12 +535,22 @@ fprintf(stderr,"png read error: out of memory..\n");
               ptr = (long)((double)i*rheight/vheight)*rwidth
                      + (long)((double)j*rwidth/vwidth);
               ptr *= 3;
+              if (components == 4){
+                ptr *= 4;
+              }
             }
             tr1 = buf[ptr++];
             tg1 = buf[ptr++];
             tb1 = buf[ptr++];
             if (components == 4){
-              ptr++;
+              if( buf[ptr++]){
+                ((unsigned char*)data2)[ptr3] |= (1<< (j%8));
+              }
+              if (j%8 == 7){
+                ptr3++;
+              }else if (j == vwidth -1){
+                ptr3++;
+              }
             }
           }else{
             ptr = (long)((double)i*rheight/vheight)*rwidth
@@ -534,8 +620,88 @@ fprintf(stderr,"png read error: out of memory..\n");
       image->data = NULL;
     }
     XDestroyImage(image);
+
+    if (image_mask != NULL){
+      image_mask->data = data2;
+      GC gc2;
+      XGCValues values;
+
+      values.foreground = 1;
+      values.background = 0;
+      Pixmap pix = XCreatePixmap(display,window, vwidth,vheight,1);
+      gc2 = XCreateGC(display, pix, GCForeground | GCBackground, &values);
+
+      *pixmap_mask = pix;
+      XPutImage(display,pix,gc2,image_mask,0,0,0,0,vwidth,vheight);
+      if (image_mask->data != NULL){
+        delete image_mask->data;
+        image_mask->data = NULL;
+      }
+      XFreeGC(display,gc2);
+      XDestroyImage(image_mask);
+    }
+
+
     png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
 
     fclose(ifile);
     return 0;
 }
+
+void png_write_fn(png_structp png_ptr, png_bytep buf, png_size_t len) {
+  FILE* fp = (FILE*)png_get_io_ptr(png_ptr);
+  int ret = fwrite(buf, 1, len, fp);
+//printf("ret=%d len=%d\n",ret,len);
+  if (ret < (int)len) {
+     png_error(png_ptr, "Write error");
+  }
+}
+ 
+void png_flush_fn(png_structp png_ptr) {
+}
+ 
+long PngWriteBufToFp(FILE* fp, char* buf,long width,long height) {
+  png_structp png_ptr = png_create_write_struct(
+       PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
+  if (!png_ptr) {
+    return WS_ERR;
+  }
+ 
+  if (setjmp(png_jmpbuf(png_ptr))) {
+    png_destroy_write_struct(&png_ptr, NULL);
+    return WS_ERR;
+  }
+ 
+  png_infop info_ptr = png_create_info_struct(png_ptr);
+  if (!info_ptr) {
+    png_destroy_write_struct(&png_ptr, NULL);
+    return WS_ERR;
+  }
+ 
+  png_set_write_fn(png_ptr, fp, png_write_fn, png_flush_fn);
+ 
+  long color_bpp = 3;
+ 
+  png_set_IHDR( png_ptr, info_ptr, width, height, 8, PNG_COLOR_TYPE_RGB,
+                PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_DEFAULT,
+                PNG_FILTER_TYPE_DEFAULT);
+ 
+  void** png_row_ptrs = (void**)png_malloc(png_ptr, sizeof(void*) * height);
+  for (int i = 0; i < height; ++i) {
+    png_row_ptrs[i] = png_malloc(png_ptr, color_bpp * width);
+    memcpy(png_row_ptrs[i], buf, color_bpp * width);
+    buf += width * color_bpp;      
+  }
+
+  png_set_rows(png_ptr, info_ptr, (png_bytepp)png_row_ptrs);
+  info_ptr->valid |= PNG_INFO_IDAT;
+  png_write_png(png_ptr, info_ptr, PNG_TRANSFORM_IDENTITY, NULL);
+ 
+  for (int i = 0; i < height; ++i) {
+    png_free(png_ptr, png_row_ptrs[i]);
+  }
+  png_free(png_ptr, png_row_ptrs);
+  png_destroy_write_struct(&png_ptr, &info_ptr);
+
+  return WS_NO_ERR;
+}
