@@ -218,8 +218,8 @@ WSMFtrace("WSDwinKeyboard::setFepPos() setfl=0x%x\n",lf);
 }
 
 WSCbool WSDwinKeyboard::_dispatch_key(MSG* msg){
-  if (msg->wParam ==VK_PROCESSKEY){
-//printf("process key ev=%d\n",msg->message);
+  if (msg->message == WM_KEYDOWN && msg->wParam ==VK_PROCESSKEY){
+printf("process key ev=%d\n",msg->message);
     if (WSGIappLocaleSet()->getSystemLocaleEncoding()==WS_EN_EUCKR &&
         _fep_dev != NULL && msg->message != WM_KEYUP){
       setFepPos(_fep_dev,_fep_pos_x,_fep_pos_y,_fep_font_no,_fep_fg_no,_fep_bg_no);
@@ -230,13 +230,13 @@ WSCbool WSDwinKeyboard::_dispatch_key(MSG* msg){
     }
     return False;
   }
-//printf("here001 ev=%d code=0x%x\n",msg->message,msg->wParam);
+printf("here001 ev=%d code=0x%x\n",msg->message,msg->wParam);
   if (msg->message == WM_CHAR || msg->message == WM_SYSCHAR){
     setIsPressed(True);
-//printf("here001 code=0x%x\n",msg->wParam);
+printf("here001 code=0x%x\n",msg->wParam);
     if (GetKeyState(VK_CONTROL) <0){
       if (msg->message == WM_CHAR ){
-//printf("here001 return..1\n");
+printf("here001 return..1\n");
         return True;
       }
       return False;
@@ -264,12 +264,12 @@ WSCbool WSDwinKeyboard::_dispatch_key(MSG* msg){
         ((msg->wParam & 0x80 && msg->wParam < 0xff) && 
         WSGIappLocaleSet()->getSystemLocaleEncoding()==WS_EN_EUCCN)){
           _pos = 1;
-//printf("here001 return..2\n");
+printf("here001 return..2\n");
           setKey(0);
           return True;
         }else{
 WSMFtrace("key pressed key=#%s#  XXZZk1 xx\n",_buf);
-//printf("key pressed key=#%s#  XXZZk1 xx\n",_buf);
+printf("key pressed key=#%s#  XXZZk1 xx\n",_buf);
           setText(_buf,WS_EN_LOCALE);
           _is_tty_function = False;
           if (_buf[0] == '!'){
@@ -401,7 +401,7 @@ WSMFtrace("key pressed key=#%s#  XXZZk1 xx\n",_buf);
 //          _buf[0] = 0;
 //        }
 WSMFtrace("key pressed key=#%s#  XXZZk1 xx2\n",_buf);
-//printf("key pressed2 key=#%s#  XXZZk1 xx2 pos=%d\n",_buf,_pos);
+printf("key pressed2 key=#%s#  XXZZk1 xx2 pos=%d\n",_buf,_pos);
         setText(_buf,WS_EN_LOCALE);
         _pos = 0;
         _is_tty_function = False;
@@ -435,13 +435,13 @@ WSMFtrace("key pressed shift=%d alt=%d XXZZk1\n",_with_shift,_with_alt);
     }
   }else if (msg->message == WM_KEYDOWN){
     setIsPressed(True);
-//printf("keydown key=0x%x  encode=%d\n",msg->wParam,WSGIappLocaleSet()->getSystemLocaleEncoding());
-//printf("here01 0x%x 0x%x\n",msg->wParam,msg->lParam);
+printf("keydown key=0x%x  encode=%d\n",msg->wParam,WSGIappLocaleSet()->getSystemLocaleEncoding());
+printf("here01 0x%x 0x%x\n",msg->wParam,msg->lParam);
 WSMFtrace("key down setkey=0x%x  XXZZk141\n",msg->wParam);
     if (GetKeyState(VK_CONTROL) < 0){
 WSMFtrace("key pressed key=#%s#  XXZZk12\n",_buf);
       long key1 = (long)msg->wParam;
-//printf("key1=0x%x %d\n",key1,key1);
+printf("key1=0x%x %d\n",key1,key1);
       if ( msg->wParam == VK_LEFT ||
            msg->wParam == VK_RIGHT ||
            msg->wParam == VK_UP ||
@@ -467,7 +467,7 @@ WSMFtrace("key pressed key=#%s#  XXZZk12\n",_buf);
            msg->wParam == VK_SNAPSHOT ||
            (msg->wParam > VK_F1 -1 && msg->wParam < VK_F12 +1)){
         long key1 = (long)msg->wParam;
-//printf("key2=0x%x %d\n",key1,key1);
+printf("key2=0x%x %d\n",key1,key1);
         setKey(key1);
       }else
       if (key1 == '!'){
@@ -591,7 +591,7 @@ WSMFtrace("key pressed key=#%s#  XXZZk12\n",_buf);
         setKey(key1);
       }
 WSMFtrace("key pressed setkey=0x%x  XXZZk12\n",msg->wParam);
-//printf("key pressed executed2.. setkey=0x%x  #%s# encode=%d\n",msg->wParam,_buf,WSGIappLocaleSet()->getSystemLocaleEncoding());
+printf("key pressed executed2.. setkey=0x%x  #%s# encode=%d\n",msg->wParam,_buf,WSGIappLocaleSet()->getSystemLocaleEncoding());
       _buf[0] = 0;
       setText(_buf,WS_EN_LOCALE);
       _with_cntl = True;
@@ -632,7 +632,7 @@ WSMFtrace("key pressed setkey=0x%x  XXZZk12\n",msg->wParam);
         return True;
       }
     }else{
-//printf("here0\n");
+printf("here0\n");
       if ( msg->wParam == VK_LEFT ||
            msg->wParam == VK_RIGHT ||
            msg->wParam == VK_UP ||
@@ -658,7 +658,7 @@ WSMFtrace("key pressed setkey=0x%x  XXZZk12\n",msg->wParam);
            msg->wParam == VK_SNAPSHOT ||
            (msg->wParam > VK_F1 -1 && msg->wParam < VK_F12 +1)){
         long key1 = (long)msg->wParam;
-//printf("key2=0x%x %d\n",key1,key1);
+printf("key2=0x%x %d\n",key1,key1);
         setKey(key1);
   
 WSMFtrace("key pressed setkey=0x%x  XXZZk14\n",msg->wParam);
@@ -695,7 +695,7 @@ WSMFtrace("key pressed setkey=0x%x  XXZZk14\n",msg->wParam);
         }else{
           _is_tty_function = False;
         }
-//printf("executed3.. buf=#%s#\n",_buf);
+printf("executed3.. buf=#%s#\n",_buf);
         WSCbool dispatched = executeHandler(True);
         if (dispatched != False){
           return False;
@@ -704,7 +704,7 @@ WSMFtrace("key pressed setkey=0x%x  XXZZk14\n",msg->wParam);
         }
       }else{
         long key1 = (long)msg->wParam;
-//printf("key3=0x%x %d\n",key1,key1);
+printf("key3=0x%x %d\n",key1,key1);
         if (key1 == '!'){
           setKey(WSK_exclam);
         }else
@@ -810,7 +810,7 @@ WSMFtrace("key pressed setkey=0x%x  XXZZk14\n",msg->wParam);
           setKey(key1);
         }
 WSMFtrace("key pressed setkey=0x%x  XXZZk142\n",msg->wParam);
-//printf("key pressed4 setkey=0x%x  XXZZk142\n",msg->wParam);
+printf("key pressed4 setkey=0x%x  XXZZk142\n",msg->wParam);
         if ( msg->wParam == WSK_Left ||
              msg->wParam == WSK_Up ||
              msg->wParam == WSK_Right ||
@@ -874,7 +874,7 @@ WSMFtrace("key pressed setkey=0x%x  XXZZk142\n",msg->wParam);
   }else if (msg->message == WM_SYSKEYDOWN){
     setIsPressed(True);
     long key1 = (long)msg->wParam;
-//printf("key6=0x%x %d\n",key1,key1);
+printf("key6=0x%x %d\n",key1,key1);
     if (key1 == '!'){
       setKey(WSK_exclam);
     }else
