@@ -896,31 +896,56 @@ long WSDwindraw::drawStretchedImage(short x,short y,WSCushort w,WSCushort h,WSDi
     return WS_ERR;
   }
   HBITMAP pix = (HBITMAP)image->getValue1();
+  HBITMAP pix2 = (HBITMAP)image->getValue3();
   if (pix == 0){
     return WS_NO_ERR;
   }
   long icon = (long)image->getValue2();
   if (icon == 0){
     HDC mdc = CreateCompatibleDC(_hdc);
-
+    HDC mdc2 = 0;
+    
     HPALETTE pl = WSGIwinAppDev()->getPalette();
     HPALETTE plbk = 0;
+    HPALETTE plbk2 = 0;
     if (pl != 0){
       plbk = SelectPalette(mdc,pl,FALSE);
       RealizePalette(mdc);
+      if (pix2 != 0){
+        plbk2 = SelectPalette(mdc2,pl,FALSE);
+        RealizePalette(mdc2);
+      }
     }
     HBITMAP pxbk = (HBITMAP)SelectObject(mdc,(void*)pix);
     SetMapMode(mdc,GetMapMode(_hdc));
+
+    HBITMAP pxbk2 = (HBITMAP)0;
+    if (pix2 != 0){
+      pxbk2 = (HBITMAP)SelectObject(mdc2,(void*)pix2);
+      SetMapMode(mdc2,GetMapMode(_hdc));
+    }
+
     WSCulong iw = image->getImageWidth();
     WSCulong ih = image->getImageHeight();
     SetStretchBltMode(_hdc,STRETCH_DELETESCANS);
-    BOOL fl = StretchBlt(_hdc,x,y,w,h,mdc,0,0,iw,ih,SRCCOPY);
+    if (pix2 == 0){
+      StretchBlt(_hdc,x,y,w,h,mdc,0,0,iw,ih,SRCCOPY);
+    }else{
+      StretchBlt(_hdc,x,y,w,h,mdc2,0,0,iw,ih,SRCAND);
+      StretchBlt(_hdc,x,y,w,h,mdc,0,0,iw,ih,SRCPAINT);
+    }
     if (pl != 0){
       SelectPalette(mdc,plbk,FALSE);
+      if (pix2 != 0){
+        SelectPalette(mdc2,plbk2,FALSE);
+      }
     }
     SelectObject(mdc,pxbk);
     DeleteDC(mdc);
-
+    if (pix2 != 0){
+      SelectObject(mdc2,pxbk2);
+      DeleteDC(mdc2);
+    }
   }else{
     DrawIconEx(_hdc,x,y,(HICON)pix,w,h,0,NULL,DI_NORMAL);
   }
@@ -932,9 +957,13 @@ long WSDwindraw::drawImage(short x,short y,WSCushort w,WSCushort h,WSDimage* ima
     return WS_ERR;
   }
   HBITMAP pix = (HBITMAP)image->getValue1();
+  HBITMAP pix2 = (HBITMAP)image->getValue3();
   if (pix == 0){
     return WS_NO_ERR;
   }
+  if (pix2 == (HBITMAP)-1){
+    pix2 = 0;
+  }
   long icon = (long)image->getValue2();
 
   POINT p;
@@ -993,15 +1022,28 @@ long WSDwindraw::drawImage(short x,short y,WSCushort w,WSCushort h,WSDimage* ima
     WSGFandArea(dx1,dy1,dw1,dh1,x+xoff,y+yoff,pw,ph,&dx,&dy,&dw,&dh);
 
     HDC mdc = CreateCompatibleDC(_hdc);
-
+    HDC mdc2 = 0;
+    if (pix2 != 0){
+      mdc2 = CreateCompatibleDC(_hdc);
+    }
     HPALETTE pl = WSGIwinAppDev()->getPalette();
     HPALETTE plbk = 0;
+    HPALETTE plbk2 = 0;
     if (pl != 0){
       plbk = SelectPalette(mdc,pl,FALSE);
       RealizePalette(mdc);
+      if (mdc2 != 0){
+        plbk2 = SelectPalette(mdc2,pl,FALSE);
+        RealizePalette(mdc2);
+      }
     }
     HBITMAP pxbk = (HBITMAP)SelectObject(mdc,(void*)pix);
+    HBITMAP pxbk2 = (HBITMAP)0;
     SetMapMode(mdc,GetMapMode(_hdc));
+    if (pix2 != 0){
+      pxbk2 = (HBITMAP)SelectObject(mdc2,(void*)pix2);
+      SetMapMode(mdc2,GetMapMode(_hdc));
+    }
     if (_printer_hdc != 0){
       x = (short)((x - _pr_x) * _pr_scale_x);
       y = (short)((y - _pr_y) * _pr_scale_y);
@@ -1014,15 +1056,32 @@ long WSDwindraw::drawImage(short x,short y,WSCushort w,WSCushort h,WSDimage* ima
       SetStretchBltMode(_hdc,STRETCH_DELETESCANS);
       WSCulong iw = image->getImageWidth();
       WSCulong ih = image->getImageHeight();
-      BOOL fl = StretchBlt(_hdc,dx,dy,dw,dh,mdc,dx -x -xoff,dy-y-yoff,iw,ih,SRCCOPY);
+      if (pix2 == 0){
+        StretchBlt(_hdc,dx,dy,dw,dh,mdc,dx -x -xoff,dy-y-yoff,iw,ih,SRCCOPY);
+      }else{
+        StretchBlt(_hdc,dx,dy,dw,dh,mdc2,dx -x -xoff,dy-y-yoff,iw,ih,SRCAND);
+        StretchBlt(_hdc,dx,dy,dw,dh,mdc,dx -x -xoff,dy-y-yoff,iw,ih,SRCPAINT);
+      }
     }else{
-      BitBlt(_hdc,dx,dy,dw,dh,mdc,dx -x -xoff,dy -y -yoff,SRCCOPY);
+      if (pix2 == 0){
+        BitBlt(_hdc,dx,dy,dw,dh,mdc,dx -x -xoff,dy -y -yoff,SRCCOPY);
+      }else{
+        BitBlt(_hdc,dx,dy,dw,dh,mdc2,dx -x -xoff,dy -y -yoff,SRCAND);
+        BitBlt(_hdc,dx,dy,dw,dh,mdc,dx -x -xoff,dy -y -yoff,SRCPAINT);
+      }
     }
     if (pl != 0){
       SelectPalette(mdc,plbk,FALSE);
+      if (pix2 != 0){
+        SelectPalette(mdc2,plbk2,FALSE);
+      }
     }
     SelectObject(mdc,pxbk);
     DeleteDC(mdc);
+    if (pix2 != 0){
+      SelectObject(mdc2,pxbk2);
+      DeleteDC(mdc2);
+    }
 
   }else{
     short clx_bk = _clip_x;
@@ -1058,25 +1117,42 @@ long WSDwindraw::drawImage(short x,short y,short sx,short sy,WSCushort w,WSCusho
     return WS_ERR;
   }
   HBITMAP pix = (HBITMAP)image->getValue1();
+  HBITMAP pix2 = (HBITMAP)image->getValue3();
 
   if (pix == (HBITMAP)-1 || pix == (HBITMAP)0){
     return WS_NO_ERR;
   }
+
   long dx,dy;
   WSCulong dw,dh;
   WSGFandArea(x,y,w,h,_clip_x,_clip_y,_clip_w,_clip_h,&dx,&dy,&dw,&dh);
 
   HDC mdc = CreateCompatibleDC(_hdc);
+  HDC mdc2 = 0;
+  HPALETTE plbk2 = 0;
+  if (pix2 != 0){
+    mdc2 = CreateCompatibleDC(_hdc);
+  }
 
   HPALETTE pl = WSGIwinAppDev()->getPalette();
   HPALETTE plbk = 0;
   if (pl != 0){
     plbk = SelectPalette(mdc,pl,FALSE);
     RealizePalette(mdc);
+    if (pix2 != 0){
+      plbk2 = SelectPalette(mdc2,pl,FALSE);
+      RealizePalette(mdc2);
+    }
   }
+
+
   HBITMAP pxbk = (HBITMAP)SelectObject(mdc,(void*)pix);
   SetMapMode(mdc,GetMapMode(_hdc));
-
+  HBITMAP pxbk2 = (HBITMAP)0;
+  if (pix2 !=0){
+    pxbk2 = (HBITMAP)SelectObject(mdc2,(void*)pix2);
+    SetMapMode(mdc2,GetMapMode(_hdc));
+  } 
   if (_printer_hdc != 0){
     x = (short)((x - _pr_x) * _pr_scale_x);
     y = (short)((y - _pr_y) * _pr_scale_y);
@@ -1089,9 +1165,19 @@ long WSDwindraw::drawImage(short x,short y,short sx,short sy,WSCushort w,WSCusho
     SetStretchBltMode(_hdc,STRETCH_DELETESCANS);
     WSCulong iw = image->getImageWidth();
     WSCulong ih = image->getImageHeight();
-    BOOL fl = StretchBlt(_hdc,dx,dy,dw,dh,mdc,dx -x +sx,dy -y +sy,iw,ih,SRCCOPY);
+    if (pix2 == 0){
+      StretchBlt(_hdc,dx,dy,dw,dh,mdc,dx -x +sx,dy -y +sy,iw,ih,SRCCOPY);
+    }else{
+      StretchBlt(_hdc,dx,dy,dw,dh,mdc2,dx -x +sx,dy -y +sy,iw,ih,SRCAND);
+      StretchBlt(_hdc,dx,dy,dw,dh,mdc,dx -x +sx,dy -y +sy,iw,ih,SRCPAINT);
+    }
   }else{
-    BitBlt(_hdc,dx,dy,dw,dh,mdc,dx -x +sx,dy -y +sy,SRCCOPY);
+    if (pix2 == 0){
+      BitBlt(_hdc,dx,dy,dw,dh,mdc,dx -x +sx,dy -y +sy,SRCCOPY);
+    }else{
+      BitBlt(_hdc,dx,dy,dw,dh,mdc2,dx -x +sx,dy -y +sy,SRCAND);
+      BitBlt(_hdc,dx,dy,dw,dh,mdc,dx -x +sx,dy -y +sy,SRCPAINT);
+    }
   }
 //printf("windraw::drawImage sx,sy=%d,%d dx,dy=%d,%d\n",sx,sy,dx,dy);
 //printf("windraw::drawImage %d,%d %d,%d 0x%x\n",dx -x +sx,dy -y +sy,dw,dh,pix);  
@@ -1099,10 +1185,17 @@ long WSDwindraw::drawImage(short x,short y,short sx,short sy,WSCushort w,WSCusho
 
   if (pl != 0){
     SelectPalette(mdc,plbk,FALSE);
+    if (pix2 != 0){
+      SelectPalette(mdc2,plbk2,FALSE);
+    }
   }
   SelectObject(mdc,pxbk);
   DeleteDC(mdc);
 
+  if (pix2 != 0){
+    SelectObject(mdc2,pxbk2);
+    DeleteDC(mdc2);
+  }
   return WS_NO_ERR;
 }
 
@@ -1418,7 +1511,7 @@ static WSDcolor* color = NULL;
   WSCulong clw,clh;
   WSGFandArea(x,y,w,h,_clip_x,_clip_y,_clip_w,_clip_h,&clx,&cly,&clw,&clh);
 
-setForeColor(c2);
+  setForeColor(c2);
   drawFillRect(x,y,w,h);
 
   WSDcolor* col1 = WSGIappColorSet()->getColor(c1);
@@ -1428,9 +1521,10 @@ setForeColor(c2);
   long r2,g2,b2;
   long r3,g3,b3;
   if (col1 == NULL || col2 == NULL || col3 == NULL){
-    setRegion(clx_bk,cly_bk,clw_bk,clh_bk);
+//    setRegion(clx_bk,cly_bk,clw_bk,clh_bk);
     return WS_ERR;
   }
+  setRegion(clx,cly,clw,clh);
   fgcolor = -100;
   bgcolor = -100;
   _fg_color = 0;
