@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2002 Toshikaz Hirabayashi
+// Copyright (C) 1999-2004 Toshikaz Hirabayashi
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -39,6 +39,9 @@
 #include <../../sys/pixmaps/bi18.xpm>
 #include <../../sys/pixmaps/bi19.xpm>
 #include <../../sys/pixmaps/endbtn.xpm>
+#include <../../sys/pixmaps/bi40.xpm>
+#include <../../sys/pixmaps/bi41.xpm>
+#include <../../sys/pixmaps/bi42.xpm>
 
 #include <../../sys/pixmaps/bi4.xpm>
 #include <../../sys/pixmaps/bi23.xpm>
@@ -125,6 +128,9 @@ WSGFsetTimePoint();
    WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/bi13.xpm",bi13_xpm);
    WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/bi15.xpm",bi15_xpm);
    WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/bi16.xpm",bi16_xpm);
+   WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/bi40.xpm",bi40_xpm);
+   WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/bi41.xpm",bi41_xpm);
+   WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/bi42.xpm",bi42_xpm);
    WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/bi17.xpm",bi17_xpm);
    WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/bi18.xpm",bi18_xpm);
    WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/bi19.xpm",bi19_xpm);
@@ -154,7 +160,6 @@ WSGFsetTimePoint();
    WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/mrr.xpm",mrr_xpm);
    WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/mrl.xpm",mrl_xpm);
 
-   xapp->execInitHandler();
 //   WSDxcolor::restart_blink_server();
 
    extern void _xfonts_init_();
@@ -173,9 +178,11 @@ int WSGFwsInitialize(){
 }
 
 int main(int argc,char** argv){
+  _dbprintf_init();
   main1(argc,argv);
   WSDappDev* app = WSGIappDev();
   WSDxappDev* xapp = (WSDxappDev*)app->cast("WSDxappDev");
+  xapp->execInitHandler();
   xapp->handleEvents();
   return 0;
 }
