@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2002 Toshikaz Hirabayashi
+// Copyright (C) 1999-2004 Toshikaz Hirabayashi
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -28,6 +28,7 @@
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/Xatom.h>
+#include <X11/Shell.h>
 #include <WSCbaseList.h>
 #include <WSCbase.h>
 #include <WSCdevice.h>
@@ -184,6 +185,9 @@ long WSDxappDev::loadGuiPolicy(){
           setGuiPolicy(val,False);
         }
       }else
+      if (!strcmp((char*)index,"#EXPOLICY") && strcmp((char*)data,"")){
+        setExtGuiPolicy((char*)data,True);
+      }else
       if (!strcmp((char*)index,"#TOPSHADOWCOLOR") && strcmp((char*)data,"")){
         short cno = WSGIappColorSet()->getColorNo((char*)data);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_TOPSHADOWCOLOR,cno);
@@ -281,6 +285,8 @@ long WSDxappDev::saveGuiPolicy(){
   WSCstring str;
   str << "#POLICY\n";
   str << getGuiPolicy() << "\n";
+  str << "#EXPOLICY\n";
+  str << getExtGuiPolicyName() << "\n";
   str << "#TOPSHADOWCOLOR\n";
   str << WSGIappColorSet()->getColor(WS_DF_TOPSHADOWCOLOR)->getSrc() << "\n";
   str << "#BOTTOMSHADOWCOLOR\n";
@@ -396,9 +402,26 @@ long WSDxappDev::initialize(){
 
   _gc = 0;
   _gc2 = 0;
-  _app_widget = XtAppInitialize ( &_app_context, 
+  if (_alt_context != NULL){
+    _app_context = (XtAppContext)_alt_context;
+    Display** displays;
+    Cardinal dnum = 0;
+    XtGetDisplays(_app_context,&displays,&dnum);
+    Display* primary = NULL;
+    if (dnum < 1){
+      return WS_ERR;
+    }
+    primary = displays[0];
+    _app_widget = XtAppCreateShell(
+                (const char *)"Application", 
+                (const char *)"Application", 
+                applicationShellWidgetClass,
+                primary, (ArgList)NULL, 0 );
+  }else{
+    _app_widget = XtAppInitialize ( &_app_context, 
                 (const char *)"Application", (XrmOptionDescList)NULL, 0, 
                 &_iprm, _cprm,(char**)NULL, (ArgList)NULL, 0 );
+  }
   _disp = XtDisplay(_app_widget);
 
   XSetErrorHandler(_ehandler);
@@ -425,27 +448,35 @@ long WSDxappDev::dispatchEvent(){
 long WSDxappDev::processEvent(){
   return _dispatchEvent(False);
 }
-long WSDxappDev::_dispatchEvent(WSCbool wait){
+long WSDxappDev::_dispatchEvent(WSCbool wait,XEvent* ev){
+dbprintf("WSDxappDev::_dispatchEvent %s:%d start\n",__FILE__,__LINE__);
 
 static long cnt = 0;
 cnt++;
   XEvent e;
-  if (wait != False){
-    XtAppNextEvent(_app_context,&e);
-  }else{
-    Boolean fl = XtAppPeekEvent(_app_context,&e);
-    if (fl == 0){
+  if (ev == NULL){
+    if (wait != False){
+      XtAppNextEvent(_app_context,&e);
+    }else{
+      Boolean fl = XtAppPeekEvent(_app_context,&e);
+      if (fl == 0){
+        return WS_NO_ERR;
+      }
+    }
+
+#ifndef KINPUT2
+    if (XFilterEvent(&e,None)){
+dbprintf("WSDxappDev::_dispatchEvent %s:%d done1\n",__FILE__,__LINE__);
       return WS_NO_ERR;
     }
+#endif
+
+  }else{
+    memcpy(&e,ev,sizeof(e));
   }
   WSCbool dispatch = True; 
 
-#ifndef KINPUT2
-  if (XFilterEvent(&e,None)){
-    return WS_NO_ERR;
-  }
-#endif
-
+dbprintf("WSDxappDev::_dispatchEvent %s:%d ev=%d\n",__FILE__,__LINE__,e.type);
   if (e.type == ClientMessage){
 //printf("client message !! here! xappdev \n");
     if (_accept_atom_for_sock == 0){
@@ -462,6 +493,7 @@ cnt++;
       if (hd != NULL){
         hd(data.sock,(void*)data.socket,data.addr);
       }
+dbprintf("WSDxappDev::_dispatchEvent %s:%d done2\n",__FILE__,__LINE__);
       return WS_NO_ERR;
     }
     if (e.xclient.message_type == _read_event_for_execute){
@@ -474,6 +506,7 @@ cnt++;
 //printf("client message !! here! execute exec_read\n");
         WSDunixExecute::_callback(obj);
       }
+dbprintf("WSDxappDev::_dispatchEvent %s:%d done3\n",__FILE__,__LINE__);
       return WS_NO_ERR;
     }
   }
@@ -482,6 +515,7 @@ cnt++;
     setSelectionOwner(NULL);
     WSGIappObjectList()->execInitialize();
     WSGIappObjectList()->execUpdate();
+dbprintf("WSDxappDev::_dispatchEvent %s:%d done4\n",__FILE__,__LINE__);
     return WS_NO_ERR;
   }
 
@@ -556,6 +590,7 @@ static Atom prop4 = 0;
     }
 
     XSendEvent(_disp,se.requestor,False,0,(XEvent*)&se);
+dbprintf("WSDxappDev::_dispatchEvent %s:%d done5\n",__FILE__,__LINE__);
     return WS_NO_ERR;
   }
   if (e.type == SelectionNotify){
@@ -600,6 +635,8 @@ static Atom prop4 = 0;
         }
         WSGIxkeyboard()->WSDkeyboard::setSelectedString(buffer,
           WSGIappLocaleSet()->getSystemLocaleEncoding());
+//printf("type=%d fmt=%d left=%d\n",type,fmt,left);
+//printf("xappdev:: comming data by cut buffer...#%s# %d\n",buffer, WSGIappLocaleSet()->getSystemLocaleEncoding());
 //        WSGIxkeyboard()->WSDkeyboard::setSelectedString(buffer);
 //printf("xappdev:: comming data3...type=%d %d nitems=%d #%s#\n",type,fmt,nitems,buf);
       }else{
@@ -668,11 +705,12 @@ static Atom prop4 = 0;
     WSGIappMouse()->setStatus(state);
   }
 
-  if (dispatch == True){
+  if (dispatch == True && ev == NULL){
     XtDispatchEvent(&e);
   }
   WSGIappObjectList()->execInitialize();
   WSGIappObjectList()->execUpdate();
+dbprintf("WSDxappDev::_dispatchEvent %s:%d done6\n",__FILE__,__LINE__);
   return WS_NO_ERR;
 }
 
