@@ -28,13 +28,41 @@ int xunicode_enable = False;
 static const char* xu_lang = NULL;
 static char xu_language[3] = {0, 0, 0};
 
-#define DEFAULT_CHAR_SET \
+#define DEFAULT_CHAR_SET_BIG5 \
+"* -kc-fixed-*-big5-0, \
+ -isas-song ti-*-gb2312.1980-0, \
+ -*-*-*-ksc5601.1987-0, \
+ -*-*-*-jisx0201.1976-0,\
+ -*-*-*-jisx0208.1983-0"
+
+#define DEFAULT_CHAR_SET_EUCKR \
+"* -*-*-*-ksc5601.1987-0, \
+ -*-*-*-jisx0201.1976-0,\
+ -*-*-*-jisx0208.1983-0, \
+ -isas-song ti-*-gb2312.1980-0, \
+ -kc-fixed-*-big5-0"
+
+#define DEFAULT_CHAR_SET_EUCJP \
 "* -*-*-*-jisx0201.1976-0,\
  -*-*-*-jisx0208.1983-0, \
  -*-*-*-ksc5601.1987-0, \
  -isas-song ti-*-gb2312.1980-0, \
  -kc-fixed-*-big5-0"
 
+#define DEFAULT_CHAR_SET_SJIS \
+"* -*-*-*-jisx0201.1976-0,\
+ -*-*-*-jisx0208.1983-0, \
+ -*-*-*-ksc5601.1987-0, \
+ -isas-song ti-*-gb2312.1980-0, \
+ -kc-fixed-*-big5-0"
+
+#define DEFAULT_CHAR_SET_EUCCN \
+"* -isas-song ti-*-gb2312.1980-0, \
+ -*-*-*-jisx0201.1976-0,\
+ -*-*-*-jisx0208.1983-0, \
+ -*-*-*-ksc5601.1987-0, \
+ -kc-fixed-*-big5-0"
+
 
 /* link list */
 static XUInfoDisplay* xu_info_display = NULL;
@@ -388,10 +416,34 @@ static void XUInitSetting()
   
   snprintf(fn, 1024, "%s/.qti18nrc", home);
   if(XUReadSetting(fn)) return;
-  char* buffer= new char[strlen(DEFAULT_CHAR_SET)+64];
-  strcpy(buffer,DEFAULT_CHAR_SET);
-  XUStripSpace(buffer);
-  XUSetSetting(buffer);
+
+  long lc = XUGetLocale();
+  if (lc == XU_CONV_EUCJP){
+    char* buffer= new char[strlen(DEFAULT_CHAR_SET_EUCJP)+64];
+    strcpy(buffer,DEFAULT_CHAR_SET_EUCJP);
+    XUStripSpace(buffer);
+    XUSetSetting(buffer);
+  }else if (lc == XU_CONV_SJIS){
+    char* buffer= new char[strlen(DEFAULT_CHAR_SET_SJIS)+64];
+    strcpy(buffer,DEFAULT_CHAR_SET_SJIS);
+    XUStripSpace(buffer);
+    XUSetSetting(buffer);
+  }else if (lc == XU_CONV_EUCKR){
+    char* buffer= new char[strlen(DEFAULT_CHAR_SET_EUCKR)+64];
+    strcpy(buffer,DEFAULT_CHAR_SET_EUCKR);
+    XUStripSpace(buffer);
+    XUSetSetting(buffer);
+  }else if (lc == XU_CONV_EUCCN){
+    char* buffer= new char[strlen(DEFAULT_CHAR_SET_EUCCN)+64];
+    strcpy(buffer,DEFAULT_CHAR_SET_EUCCN);
+    XUStripSpace(buffer);
+    XUSetSetting(buffer);
+  }else if (lc == XU_CONV_BIG5){
+    char* buffer= new char[strlen(DEFAULT_CHAR_SET_BIG5)+64];
+    strcpy(buffer,DEFAULT_CHAR_SET_BIG5);
+    XUStripSpace(buffer);
+    XUSetSetting(buffer);
+  }
 }
 
 
